@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: GPL-3.0-or-later WITH cryptsetup-OpenSSL-exception */
 /*
     This file is part of sscg.
 
@@ -14,7 +15,20 @@
     You should have received a copy of the GNU General Public License
     along with sscg.  If not, see <http://www.gnu.org/licenses/>.
 
-    Copyright 2019 by Stephen Gallagher <sgallagh@redhat.com>
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations
+    including the two.
+    You must obey the GNU General Public License in all respects
+    for all of the code used other than OpenSSL.  If you modify
+    file(s) with this exception, you may extend this exception to your
+    version of the file(s), but you are not obligated to do so.  If you
+    do not wish to do so, delete this exception statement from your
+    version.  If you delete this exception statement from all source
+    files in the program, then also delete it here.
+
+    Copyright 2019-2025 by Stephen Gallagher <sgallagh@redhat.com>
 */
 
 #ifndef _SSCG_DHPARAMS_H
