@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: GPL-3.0-or-later WITH cryptsetup-OpenSSL-exception */
 /*
     This file is part of sscg.
 
@@ -14,10 +15,22 @@
     You should have received a copy of the GNU General Public License
     along with sscg.  If not, see <http://www.gnu.org/licenses/>.
 
-    Copyright 2021 by Stephen Gallagher <sgallagh@redhat.com>
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations
+    including the two.
+    You must obey the GNU General Public License in all respects
+    for all of the code used other than OpenSSL.  If you modify
+    file(s) with this exception, you may extend this exception to your
+    version of the file(s), but you are not obligated to do so.  If you
+    do not wish to do so, delete this exception statement from your
+    version.  If you delete this exception statement from all source
+    files in the program, then also delete it here.
+
+    Copyright 2021-2025 by Stephen Gallagher <sgallagh@redhat.com>
 */
 
-#include <path_utils.h>
 #include <popt.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -30,6 +43,9 @@
 #include "include/dhparams.h"
 
 #include "config.h"
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#endif
 
 #define _GNU_SOURCE
 
@@ -69,7 +85,6 @@ set_default_options (struct sscg_options *opts)
 
   opts->lifetime = 398;
 
-  opts->dhparams_file = talloc_strdup (opts, "dhparams.pem");
   opts->dhparams_group = talloc_strdup (opts, "ffdhe4096");
   opts->dhparams_generator = 2;
 
@@ -102,10 +117,11 @@ set_default_options (struct sscg_options *opts)
 
     default:
       /* Unknown security level. Default to the highest we know about */
-      fprintf (stderr,
-               "Unknown system security level %d. Defaulting to highest-known "
-               "level.\n",
-               security_level);
+      fprintf (
+        stderr,
+        _ ("Unknown system security level %d. Defaulting to highest-known "
+           "level.\n"),
+        security_level);
       /* Fall through */
 
     case 5:
@@ -155,13 +171,13 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
   if (ret != EOK)
     goto done;
 
-  minimum_key_strength_help =
-    talloc_asprintf (tmp_ctx, "%d or larger", options->minimum_key_strength);
+  minimum_key_strength_help = talloc_asprintf (
+    tmp_ctx, _ ("%d or larger"), options->minimum_key_strength);
 
   named_groups_help =
     talloc_asprintf (tmp_ctx,
-                     "Output well-known DH parameters. The available named "
-                     "groups are: %s. (Default: \"ffdhe4096\")",
+                     _ ("Output well-known DH parameters. The available named "
+                        "groups are: %s. (Default: \"ffdhe4096\")"),
                      valid_dh_group_names (tmp_ctx));
 
   options->verbosity = SSCG_DEFAULT;
@@ -175,7 +191,7 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
       POPT_ARG_VAL,
       &options->verbosity,
       SSCG_QUIET,
-       ("Display no output unless there is an error."),
+      _ ("Display no output unless there is an error."),
       NULL
     },
 
@@ -310,7 +326,7 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
       _ ("Optional additional valid hostnames for the certificate. "
          "In addition to hostnames, this option also accepts explicit values "
          "supported by RFC 5280 such as "
-         "IP:xxx.xxx.xxx.xxx/yyy.yyy.yyy.yyy "
+         "IP:xxx.xxx.xxx.xxx "
          "May be specified multiple times."),
       _ ("alt.example.com")
     },
@@ -609,6 +625,16 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
       NULL
     },
 
+    {
+      "no-dhparams-file",
+      '\0',
+      POPT_ARG_NONE,
+      &options->skip_dhparams,
+      0,
+      _ ("Do not create the dhparams file"),
+      NULL
+    },
+
     {
       "dhparams-named-group",
       '\0',
@@ -652,7 +678,7 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
         {
         default:
           fprintf (stderr,
-                   "\nInvalid option %s: %s\n\n",
+                   _ ("\nInvalid option %s: %s\n\n"),
                    poptBadOption (pc, 0),
                    poptStrerror (opt));
           poptPrintUsage (pc, stderr, 0);
@@ -676,7 +702,7 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
     {
       if (strlen (country) != 2)
         {
-          fprintf (stderr, "Country codes must be exactly two letters.\n");
+          fprintf (stderr, _ ("Country codes must be exactly two letters.\n"));
           ret = EINVAL;
           goto done;
         }
@@ -764,10 +790,20 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
     }
   CHECK_MEM (options->hostname);
 
-  if (strnlen (options->hostname, MAXHOSTNAMELEN + 1) > MAXHOSTNAMELEN)
+  if (strnlen (options->hostname, MAX_FQDN_LEN + 1) > MAX_FQDN_LEN)
     {
       fprintf (
-        stderr, "Hostnames may not exceed %d characters\n", MAXHOSTNAMELEN);
+        stderr, _ ("FQDNs may not exceed %d characters\n"), MAX_FQDN_LEN);
+      ret = EINVAL;
+      goto done;
+    }
+
+  if ((strchr (options->hostname, '.') - options->hostname) > MAX_HOST_LEN + 4)
+    {
+      fprintf (stderr,
+               _ ("Hostnames may not exceed %d characters in Subject "
+                  "Alternative Names\n"),
+               MAX_HOST_LEN);
       ret = EINVAL;
       goto done;
     }
@@ -776,29 +812,39 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
        options struct. It's not the most efficient approach, but
        it's only done one time, so there is no sense in optimizing
        it. */
+  size_t i = 0;
   if (alternative_names)
     {
-      size_t i = 0;
       while (alternative_names[i] != NULL)
         {
           options->subject_alt_names = talloc_realloc (
-            options, options->subject_alt_names, char *, i + 2);
+            options, options->subject_alt_names, char *, i + 1);
           CHECK_MEM (options->subject_alt_names);
 
           options->subject_alt_names[i] =
             talloc_strdup (options->subject_alt_names, alternative_names[i]);
           CHECK_MEM (options->subject_alt_names[i]);
-
-          /* Add a NULL terminator to the end */
-          options->subject_alt_names[i + 1] = NULL;
           i++;
         }
     }
 
+  /*
+    The hostname must always be listed in SubjectAlternativeNames as well.
+    Note that the realloc also adds an extra entry for the NULL terminator
+  */
+  options->subject_alt_names =
+    talloc_realloc (options, options->subject_alt_names, char *, i + 2);
+  CHECK_MEM (options->subject_alt_names);
+  options->subject_alt_names[i] =
+    talloc_strdup (options->subject_alt_names, options->hostname);
+  CHECK_MEM (options->subject_alt_names[i]);
+  /* Add a NULL terminator to the end */
+  options->subject_alt_names[i + 1] = NULL;
+
   if (options->key_strength < options->minimum_key_strength)
     {
       fprintf (stderr,
-               "Key strength must be at least %d bits.\n",
+               _ ("Key strength must be at least %d bits.\n"),
                options->minimum_key_strength);
       ret = EINVAL;
       goto done;
@@ -808,27 +854,35 @@ sscg_handle_arguments (TALLOC_CTX *mem_ctx,
   options->cipher = EVP_get_cipherbyname (options->cipher_alg);
   if (!options->cipher)
     {
-      fprintf (stderr, "Invalid cipher specified: %s\n", options->cipher_alg);
+      fprintf (
+        stderr, _ ("Invalid cipher specified: %s\n"), options->cipher_alg);
       ret = EINVAL;
       goto done;
     }
 
-  /* TODO: restrict this to approved hashes.
-   * For now, we'll only list SHA[256|384|512] in the help */
   options->hash_fn = EVP_get_digestbyname (options->hash_alg);
+  if (!options->hash_fn ||
+      EVP_MD_is_a (options->hash_fn, options->hash_alg) != 1)
+    {
+      /* This check ensures that the passed hash algorithm is both a real
+       * algorithm and that it is permitted by the current system policy. */
+      fprintf (stderr, _ ("Unsupported hashing algorithm."));
+      ret = EINVAL;
+      goto done;
+    }
 
   if (!is_valid_named_group (options->dhparams_group))
     {
-      fprintf (stderr, "Unknown Diffie Hellman finite field group.\n");
+      fprintf (stderr, _ ("Unknown Diffie Hellman finite field group.\n"));
       fprintf (
-        stderr, "Valid groups are: %s.\n", valid_dh_group_names (tmp_ctx));
+        stderr, _ ("Valid groups are: %s.\n"), valid_dh_group_names (tmp_ctx));
       ret = EINVAL;
       goto done;
     }
 
   if (!options->hash_fn)
     {
-      fprintf (stderr, "Unsupported hashing algorithm.");
+      fprintf (stderr, _ ("Unsupported hashing algorithm."));
       ret = EINVAL;
       goto done;
     }
@@ -853,23 +907,23 @@ void
 print_options (struct sscg_options *opts)
 {
   size_t i = 0;
-  fprintf (stdout, "==== Options ====\n");
-  fprintf (stdout, "Certificate lifetime: %d\n", opts->lifetime);
-  fprintf (stdout, "Country: \"%s\"\n", opts->country);
-  fprintf (stdout, "State or Principality: \"%s\"\n", opts->state);
-  fprintf (stdout, "Locality: \"%s\"\n", opts->locality);
-  fprintf (stdout, "Organization: \"%s\"\n", opts->org);
-  fprintf (stdout, "Organizational Unit: \"%s\"\n", opts->org_unit);
-  fprintf (stdout, "Email Address: \"%s\"\n", opts->email);
-  fprintf (stdout, "Hostname: \"%s\"\n", opts->hostname);
+  fprintf (stdout, _ ("==== Options ====\n"));
+  fprintf (stdout, _ ("Certificate lifetime: %d\n"), opts->lifetime);
+  fprintf (stdout, _ ("Country: \"%s\"\n"), opts->country);
+  fprintf (stdout, _ ("State or Principality: \"%s\"\n"), opts->state);
+  fprintf (stdout, _ ("Locality: \"%s\"\n"), opts->locality);
+  fprintf (stdout, _ ("Organization: \"%s\"\n"), opts->org);
+  fprintf (stdout, _ ("Organizational Unit: \"%s\"\n"), opts->org_unit);
+  fprintf (stdout, _ ("Email Address: \"%s\"\n"), opts->email);
+  fprintf (stdout, _ ("Hostname: \"%s\"\n"), opts->hostname);
   if (opts->subject_alt_names)
     {
       for (i = 0; opts->subject_alt_names[i]; i++)
         {
           fprintf (stdout,
-                   "Subject Alternative Name: \"%s\"\n",
+                   _ ("Subject Alternative Name: \"%s\"\n"),
                    opts->subject_alt_names[i]);
         }
     }
-  fprintf (stdout, "=================\n");
+  fprintf (stdout, _ ("=================\n"));
 }
